@@ -20,10 +20,10 @@ class ShapeSensingFBGNeedle( sensorized_needles.FBGNeedle ):
 
         # current insertion parameters
         self.current_depth = current_depth
-        self.__current_shapetype = intrinsics.SHAPETYPE.SINGLEBEND_SINGLELAYER
-        self.current_wavelengths = -np.ones_like( self.ref_wavelengths )
-        self.current_curvatures = np.zeros( (len( sensor_location ), 2) )
-        self.insertion_parameters = { }
+        self.__current_shapetype  = intrinsics.SHAPETYPE.SINGLEBEND_SINGLELAYER
+        self.current_wavelengths  = -np.ones_like( self.ref_wavelengths )
+        self.current_curvatures   = np.zeros( (len( sensor_location ), 2) )
+        self.insertion_parameters = dict()
 
         # define needle shape-sensing optimizers
         self.optimizer = numerical.NeedleParamOptimizations(
@@ -317,10 +317,10 @@ class ShapeSensingFBGNeedle( sensorized_needles.FBGNeedle ):
             success = True
 
             if processed:
-                self.current_curvatures = self.curvatures_processed( self.current_wavelengths )
+                self.current_curvatures = 1e-3 * self.curvatures_processed( self.current_wavelengths )
 
             else:
-                self.current_curvatures = self.curvatures_raw(
+                self.current_curvatures = 1e-3 * self.curvatures_raw(
                         self.current_wavelengths, temp_comp=temp_comp )
 
         # if
@@ -454,7 +454,7 @@ class ShapeSensingMCFNeedle(ShapeSensingFBGNeedle, sensorized_needles.MCFNeedle)
             serial_number=serial_number, 
             num_channels=num_channels, 
             sensor_location=sensor_location, 
-            calibration_mat=calibration_mats,
+            calibration_mats=calibration_mats,
             weights=weights, 
             ds=ds, 
             current_depth=current_depth, 
@@ -476,11 +476,11 @@ class ShapeSensingMCFNeedle(ShapeSensingFBGNeedle, sensorized_needles.MCFNeedle)
     def from_MCFNeedle( mcfneedle: sensorized_needles.MCFNeedle, **kwargs ):
         """ Turn an FBGNeedle into a shape-sensing FBGNeedle
 
-            :param fbgneedle: FBGNeedle to turn into a sensorized one
+            :param mcfneedle: MCFNeedle to turn into a sensorized one
             :keyword ds: the ds for the ShapeSensingFBGNeedle constructor
             :keyword current_depth: the current insertion depth for the ShapeSensingFBGNeedle constructor
 
-            :return: ShapeSensingFBGNeedle with the current FBGNeedle
+            :return: ShapeSensingMCFNeedle with the current FBGNeedle
         """
         return ShapeSensingMCFNeedle(
                 mcfneedle.length,
